
#include "globals.h"
#include <math.h>
#include <time.h>

#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>
#include <gl\glaux.h>

#include "perlin.h"
#include "terrain.h"
#include "images.h"
#include "perlin.h"
#include "smoothing.h"
#include "text.h"

#pragma comment(lib, "opengl32.lib")					// link opp openGL32.lib og glu32.lib (i stede for  gjre det i prosjektet)
#pragma comment(lib, "glaux.lib")
#pragma comment(lib, "glu32.lib")

/************************************************************************
*
*	Demostrasjon av fraktalterreng ved bruk av Perlin Noise.
*
*	Laget av: J.R. Odden. Hi, Vren 2005
*	Fag: Grafisk databehandling
*	Epost: jon.r.odden@hiof.no
*	Web: http://www.sportsbil.net/skole/gb
*
*	Programmet bruker Perlin Noise til  lage hydekart for s  rendre
*   disse. Perlin Noise er bare basen, som blir brukt som argument til
*	diverse trigonometriske funksjoner som jeg har kommet frem til selv,
*	for  oppn gunstige effekter.
*	Etter Perlin Noise genereringen glattes terrenget i varierende grad,
*	delvis avhengig av hva argumentene til Perlin Noise generatoren var.
*	Dette er for  PRVE  unng helt usannsynlige terreng. Siden vi 
*	opererer med flere forskjellige variabler som alle har en tilfeldig
*	verdi, vil man kunne oppleve at terrenget blir glattet helt ut.
*	Man kan fixe dette ved  sette opp endel IF setninger men jeg har
*	ikke rukket dette.
*
*	Nr programmet starter rendrer det kun en flat mesh uten hydeverdier.
*	For  generere disse, velger du en av funksjonene ved  trykke 1, 2
*	eller 3 p tastaturet.
*
*	Under kjring av programmet har man diverse funksjoner man kan sl
*	av og p. Her er en liste over disse samt tasten man bruker for 
*	aktivere funksjonen.
*
*	Tast:		Funksjon:
*	1			Generer nytt hydekart ved bruk av funksjon 1
*	2			Generer nytt hydekart ved bruk av funksjon 2
*	3			Generer nytt hydekart ved bruk av funksjon 3
*	A			Animr en rotasjon om Y aksen
*	F			Filtrer det aktive hydekartet 10 ganger
*	S			Sl av/p rendring av skydome (himmel dome)
*	T			Sl av/p rendring av terrenget
*	W			Veksle mellom  rendre Wireframe eller Solid
*	Pil opp		Zoom inn
*	Pil ned		Zoom ut
*
*	Mus:						Funksjon:
*	Venstre knapp + bevegelse	Roter om Y aksen
*	Hyre knapp + bevegelse		Roter om X aksen
*
*
*	Funksjon 1 = noise(x, y)
*	Funksjon 2 = sin(x + 1 / |noise(x, y)|)
*	Funksjon 3 = cos(PI *(x*xC) + (y*yC) * nP * noise(x, y))
*
*	Funksjon 1 tenderer til  rendre ganske kaotiske terreng.
*	Funksjon 2 tenderer til  rendre dype daler og hye topper.
*	Funksjon 3 tenderer til  rendre litt mykere/lavere terreng,
*			   eventuelt litt hyere terreng som begynner lavt og
*			   bygger seg opp mot et hjrne eller en side.
*
*************************************************************************/

HDC			hDC=NULL;
HGLRC		hRC=NULL;
HWND		hWnd=NULL;
HINSTANCE	hInstance;

//muskontroll variabler
int currMouseButton;
int currMouseX, currMouseY;
int prevMouseX, prevMouseY;
float xv=0.0f;
float yv=0.0f;
float mouseSensitivity = 0.1;

//perlin noise variabler
float freq;
float pers;
float smoothFactor;
int smoothCount;

//diverse state variabler ++
int			listBase;
int			noiseFunc = 1;
bool		keys[256];		//tastearray
bool		active = TRUE;
bool		bRenderNewMap = FALSE;
bool		bAnimate = FALSE;
bool		fullscreen = TRUE;
bool		bRenderSky = FALSE;
bool		bRenderSolid = FALSE;
bool		bRenderTerrain = TRUE;
float		scaleValue = 0.33f;

//array som inneholder hydekartet
BYTE g_HeightMap[MAP_SIZE*MAP_SIZE];

int initGL(GLvoid)
{
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f); //hvit
	glClearDepth(1.0f);

	return TRUE;
}

GLvoid reSizeGLScene(GLsizei width, GLsizei height)
{
	//forhindre dele-p-null feil
	if (height==0)
	{
		height=1;
	}

	//reset viewport
	glViewport(0, 0, width, height);

	//velger hvilken matrise vi skal manipulere
	glMatrixMode(GL_PROJECTION);

	//resetter projeksjons matrisen
	glLoadIdentity();

	//regn ut aspect p vinduet vrt (hyde i forhold til bredde)
	gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 750.0f);

	//velger hvilken matrise vi skal manipulere
	glMatrixMode(GL_MODELVIEW);

	//resetter modelview matrisen
	glLoadIdentity();
}

bool doPerlinMap()
{
	smoothFactor = 0.0f;

	if (noiseFunc == 1) 
	{
		freq = randfloat(0.0035f, 0.015f);
		pers = randfloat(0.35f, 0.65f);
		smoothFactor = randfloat(0.2f, 0.35f);
		smoothCount = randint(1,20);
	}

	if (noiseFunc == 2)
	{
		freq = randfloat(0.005f, 0.015f);
		pers = randfloat(0.35f, 0.55f);
		smoothFactor = randfloat(0.3f, 0.45f);
		smoothCount = randint(1,10);
	}

	if (noiseFunc == 3)
	{
		freq = randfloat(0.005f, 0.009f);
		pers = randfloat(0.1f, 0.5f);
		//smoothFactor = randfloat(0.4f, 0.65f);
		smoothFactor = randfloat(0.6f, 0.85f);
		smoothCount = randint(1,10);
	}

	makePerlinMap(freq, pers, 6, noiseFunc);

	//glatt terrenget et tilfeldig antall ganger mellom 15 og 30
	for (int i=0;i<smoothCount;i++)
	{
		//bruk litt tilfeldige glatteverdier...
		if ((freq <= 0.005) && (freq >= 0.0035)) 
		{
			filterHeightField(g_HeightMap, smoothFactor);
		} 
		else if ((freq <= 0.0055) && (freq >= 0.0035))
		{
			filterHeightField(g_HeightMap, smoothFactor);
		}
		else if ((freq <= 0.0035) && (freq >= 0.0075))
		{
			filterHeightField(g_HeightMap, smoothFactor);
		}
		else if ((freq <= 0.0075) && (freq >= 0.01))
		{
			filterHeightField(g_HeightMap, smoothFactor);
		}
		else if ((freq <= 0.01) && (freq >= 0.015))
		{
			filterHeightField(g_HeightMap, smoothFactor);
		}
	}

	//lagre bildet hvis vi nsker  bruke det senere...
	//SaveAsRawFile("perlinmap.raw", MAP_SIZE*MAP_SIZE, g_HeightMap);

	return true;
}

//all tegning gjres her
int drawGLScene(GLvoid)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	listBase = createBitmapFont("Verdana", 18);

	//skriv ut tekst
	char str[100];

	glTranslatef(0.0f, 0.0f, -1.0f);
	glColor3f(0.0f, 0.0f, 1.0f);
	
	glRasterPos2f(-0.53f, 0.38f);	
	sprintf(str, "frekvens: %1.4f", freq);
	printString(listBase, str);

	glRasterPos2f(-0.53f, 0.35f);	
	sprintf(str, "persistans: %1.4f", pers);
	printString(listBase, str);

	glRasterPos2f(-0.53f, 0.32f);	
	sprintf(str, "glattingsfaktor: %1.4f", smoothFactor);
	printString(listBase, str);

	glRasterPos2f(-0.53f, 0.29f);	
	sprintf(str, "glattet: %d ganger", smoothCount);
	printString(listBase, str);

	killFont();

	//kamera posisjon og hvor kameraet skal se..
	// 			 position	      view		up vector
	gluLookAt(212, 70, 194,  186, 65, 171,  0, 1, 0);

	//skaler terrenget vrt
	glScalef(scaleValue, scaleValue * HEIGHT_RATIO, scaleValue);

	if (bRenderNewMap)
	{
		//laster inn heightmap fra en fil
		//loadRawFile("perlin.raw", MAP_SIZE*MAP_SIZE, g_HeightMap);

		//genererer heightmap vha perlin noise
		doPerlinMap();

		bRenderNewMap = false;
	}

	if (bRenderSky)
	{
		//generering av data for himmeldomen
		setSkyDomeData(MAP_SIZE/2, 16.0f, 5.0f, 180.0f);

		//textur til himmelen
		//textureSky = getTextureKoord(2.0f);

		//roter s himmelen blir OVER landskapet...
		glRotatef(270.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(90.0f, 0.0f, 0.0f, 1.0f);

		//tegn himmeldomen
		glColor3f(0.45f, 0.55f, 0.8f); //blaktig farge
		drawSky();

		//roter tilbake igjen (s ikke landskapet str 90 grader vertikalt p himmeldomen...)
		glRotatef(-270.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
	}

	if (bRenderTerrain)
	{
		glPointSize(15.0f);
		glColor3f(0.0f, 0.0f, 1.0f);

		//animer en rotasjon hvis brukeren har trykket 'A'
		if (bAnimate)
		{
			glRotatef(yv, 0.0f, 1.0f, 0.0f);
			yv += 0.2;
		}

		//roter hvis brukeren har klikket+flytta musa
		glRotatef(xv, 1.0f, 0.0f, 0.0f);
		glRotatef(yv, 0.0f, 1.0f, 0.0f);
		
		//tegn bl prikk i origo
		/*
		glBegin(GL_POINTS);
			glVertex3f(0.0f, 0.0f, 0.0f);
		glEnd();
		*/

		//flytt origo
		glTranslatef(-250.0, 0.0, -250.0);


		//tegn en bl prikk i origo etter translasjon
		/*
		glBegin(GL_POINTS);
			glVertex3f(0.0f, 0.0f, 0.0f);
		glEnd();
		*/

		//tegn terrenget
		renderHeightMap(g_HeightMap);
	}

	return TRUE;
}

GLvoid killGLWindow(GLvoid)
{
	//hvis vi er i fullscreen...
	if (fullscreen)
	{
		//g til desktop
		ChangeDisplaySettings(NULL,0);

		//og vis musepeker
		ShowCursor(TRUE);
	}

	//hvis vi har en rendering context..
	if (hRC)
	{
		//hvis vi ikke klarer  frigjre RC og DC...
		if (!wglMakeCurrent(NULL,NULL))
		{
			MessageBox(NULL,"Frigjring av RC og DC feilet..","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		//hvis vi ikke klarer  slette RC...
		if (!wglDeleteContext(hRC))
		{
			MessageBox(NULL,"Frigjring av RC feilet.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;
	}

	//hvis vi ikke klarer  frigjre DC...
	if (hDC && !ReleaseDC(hWnd,hDC))
	{
		MessageBox(NULL,"Frigjring av DC feilet.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;
	}

	//hvis vi ikke klarer  delegge vindue(t/ene)
	if (hWnd && !DestroyWindow(hWnd))
	{
		MessageBox(NULL,"Frigjring av hWnd feilet.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;
	}

	//hvis vi ikke klarer  frigjre klassen
	if (!UnregisterClass("OpenGL",hInstance))
	{
		MessageBox(NULL,"Klarte ikke  frigjre klassen.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;
	}
}

// Koden nedenfor er i hovedsak hentet fra nehe.gamedev.net + noen modifikasjoner p kontroll,
// og forandring av sprk p feilmeldingene.

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL createGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;							// Holds The Results After Searching For A Match
	WNDCLASS	wc;										// Windows Class Structure
	DWORD		dwExStyle;								// Window Extended Style
	DWORD		dwStyle;								// Window Style
	RECT		WindowRect;								// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;							// Set Left Value To 0
	WindowRect.right=(long)width;						// Set Right Value To Requested Width
	WindowRect.top=(long)0;								// Set Top Value To 0
	WindowRect.bottom=(long)height;						// Set Bottom Value To Requested Height

	fullscreen=fullscreenflag;							// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);		// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;			// WndProc Handles Messages
	wc.cbClsExtra		= 0;							// No Extra Window Data
	wc.cbWndExtra		= 0;							// No Extra Window Data
	wc.hInstance		= hInstance;					// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);	// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);	// Load The Arrow Pointer
	wc.hbrBackground	= NULL;							// No Background Required For GL
	wc.lpszMenuName		= NULL;							// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";						// Set The Class Name

	if (!RegisterClass(&wc))							// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Registrering av klassevinduet feilet.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}
	
	if (fullscreen)										// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;						// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);	// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;		// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;		// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;			// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"Fullskjerm modus er ikke stttet av\ngrafikk kortet. Kjre i vindu i stede?", "Fraktalterreng", MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;						// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Klarte ikke  lukke programet.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;							// Return FALSE
			}
		}
	}

	if (fullscreen)										// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;						// Window Extended Style
		dwStyle=WS_POPUP;								// Windows Style
		ShowCursor(FALSE);								// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;	// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;					// Windows Style
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);	// Adjust Window To True Requested Size

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,				// Extended Style For The Window
								"OpenGL",				// Class Name
								title,					// Window Title
								dwStyle |				// Defined Window Style
								WS_CLIPSIBLINGS |		// Required Window Style
								WS_CLIPCHILDREN,		// Required Window Style
								0, 0,					// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,					// No Parent Window
								NULL,					// No Menu
								hInstance,				// Instance
								NULL)))					// Dont Pass Anything To WM_CREATE
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Feil ved oppsetting av vinduet.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=					// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),					// Size Of This Pixel Format Descriptor
		1,												// Version Number
		PFD_DRAW_TO_WINDOW |							// Format Must Support Window
		PFD_SUPPORT_OPENGL |							// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,								// Must Support Double Buffering
		PFD_TYPE_RGBA,									// Request An RGBA Format
		bits,											// Select Our Color Depth
		0, 0, 0, 0, 0, 0,								// Color Bits Ignored
		0,												// No Alpha Buffer
		0,												// Shift Bit Ignored
		0,												// No Accumulation Buffer
		0, 0, 0, 0,										// Accumulation Bits Ignored
		16,												// 16Bit Z-Buffer (Depth Buffer)  
		0,												// No Stencil Buffer
		0,												// No Auxiliary Buffer
		PFD_MAIN_PLANE,									// Main Drawing Layer
		0,												// Reserved
		0, 0, 0											// Layer Masks Ignored
	};

	if (!(hDC=GetDC(hWnd)))								// Did We Get A Device Context?
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Klarte ikke  lage en GL device context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))		// Did Windows Find A Matching Pixel Format?
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Klarte ikke  finne et passende pixelformat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))			// Are We Able To Set The Pixel Format?
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Klarte ikke  sette pixelformat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))					// Are We Able To Get A Rendering Context?
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Klarte ikke  lage et GL rendering context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))						// Try To Activate The Rendering Context
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Klarte ikke  aktivere GL rendering contexten.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);							// Show The Window
	SetForegroundWindow(hWnd);							// Slightly Higher Priority
	SetFocus(hWnd);										// Sets Keyboard Focus To The Window
	reSizeGLScene(width, height);						// Set Up Our Perspective GL Screen

	if (!initGL())										// Initialize Our Newly Created GL Window
	{
		killGLWindow();									// Reset The Display
		MessageBox(NULL,"Initsialiseringen feilet.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;									// Return FALSE
	}

	return TRUE;										// Success
}


int WINAPI WinMain(	HINSTANCE	hInstance,				// Instance
					HINSTANCE	hPrevInstance,			// Previous Instance
					LPSTR		lpCmdLine,				// Command Line Parameters
					int			nCmdShow)				// Window Show State
{
	MSG		msg;										// Windows Message Structure
	BOOL	done=FALSE;									// Bool Variable To Exit Loop

	// Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Kjre fullskjerm?", "Starte i fullskjerm?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;								// Windowed Mode
	}

	// Create Our OpenGL Window
	if (!createGLWindow("Perlin Noise basert Fraktalterreng", 800, 600, 16, fullscreen))
	{
		return 0;										// Quit If Window Was Not Created
	}

	while(!done)										// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))		// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)					// Have We Received A Quit Message?
			{
				done=TRUE;								// If So done=TRUE
			}
			else										// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);					// Translate The Message
				DispatchMessage(&msg);					// Dispatch The Message
			}
		}
		else											// If There Are No Messages
		{
			// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !drawGLScene()) || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
			{
				done=TRUE;								// ESC or DrawGLScene Signalled A Quit
			}
			else if (active)							// Not Time To Quit, Update Screen
			{
				SwapBuffers(hDC);						// Swap Buffers (Double Buffering)
			}

			if (keys[VK_F1])							// Is F1 Being Pressed?
			{
				keys[VK_F1]=FALSE;						// If So Make Key FALSE
				killGLWindow();							// Kill Our Current Window
				fullscreen=!fullscreen;					// Toggle Fullscreen / Windowed Mode
				// Recreate Our OpenGL Window
				if (!createGLWindow("Perlin Noise basert Fraktalterreng", 800, 600, 16, fullscreen))
				{
					return 0;							// Quit If Window Was Not Created
				}
			}

			//trykk F for  glatte terrenget 10 ganger (trykker du to ganger er det glattet 20 ganger)
			if (keys['F'])
			{
				keys['F'] = false;
				for (int i=0;i<10;i++)
				{
					filterHeightField(g_HeightMap, randfloat(0.3f, 0.5));
				}
			}
			
			//rendrer nytt terreng med noise funksjon 1
			if (keys['1'])
			{
				keys['1'] = false;
				noiseFunc = 1;
				bRenderNewMap = true;
			}

			//rendrer nytt terreng med noise funksjon 1
			if (keys['2'])
			{
				keys['2'] = false;
				noiseFunc = 2;
				bRenderNewMap = true;
			}

			//rendrer nytt terreng med noise funksjon 1
			if (keys['3'])
			{
				keys['3'] = false;
				noiseFunc = 3;
				bRenderNewMap = true;
			}

			//sl av/p opptegning av hydekartet
			if (keys['A'])
			{
				keys['A'] = false;
				bAnimate = !bAnimate;
			}

			//sl av/p opptegning av hydekartet
			if (keys['T'])
			{
				keys['T'] = false;
				bRenderTerrain = !bRenderTerrain;
			}

			//sl av/p rendring i wireframe eller solid mode
			if (keys['W'])
			{
				keys['W'] = false;
				bRenderSolid = !bRenderSolid;
			}

			//sl av/p opptegning av himmeldomen
			if (keys['S'])
			{
				keys['S'] = false;
				bRenderSky = !bRenderSky;
			}

			//zoom inn
			if (keys[VK_UP])
			{
				scaleValue += 0.005f;
			}

			//zoom ut
			if (keys[VK_DOWN])
			{
				scaleValue -= 0.005f;
			}

		}
	}

	// Shutdown
	killGLWindow();										// Kill The Window
	return (int)(msg.wParam);								// Exit The Program
}

LRESULT CALLBACK WndProc(	HWND	hWnd,				// Handle For This Window
							UINT	uMsg,				// Message For This Window
							WPARAM	wParam,				// Additional Message Information
							LPARAM	lParam)				// Additional Message Information
{
	switch (uMsg)										// Check For Windows Messages
	{
		case WM_ACTIVATE:								// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))						// Check Minimization State
			{
				active=TRUE;							// Program Is Active
			}
			else
			{
				active=FALSE;							// Program Is No Longer Active
			}

			return 0;									// Return To The Message Loop
		}

		case WM_SYSCOMMAND:								// Intercept System Commands
		{
			switch (wParam)								// Check System Calls
			{
				case SC_SCREENSAVE:						// Screensaver Trying To Start?
				case SC_MONITORPOWER:					// Monitor Trying To Enter Powersave?
				return 0;								// Prevent From Happening
			}
			break;										// Exit
		}

		case WM_CLOSE:									// Did We Receive A Close Message?
		{
			PostQuitMessage(0);							// Send A Quit Message
			return 0;									// Jump Back
		}

		case WM_KEYDOWN:								// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;						// If So, Mark It As TRUE
			return 0;									// Jump Back
		}

		case WM_KEYUP:									// Has A Key Been Released?
		{
			keys[wParam] = FALSE;						// If So, Mark It As FALSE
			return 0;									// Jump Back
		}

		case WM_SIZE:									// Resize The OpenGL Window
		{
			reSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;									// Jump Back
		}

		case MK_LBUTTON:
		{
			//lagre x,y punktet hvor brukeren klikka for bruk senere
			prevMouseX = (int)LOWORD(lParam);
			prevMouseY = (int)HIWORD(lParam);
		}

		case WM_MOUSEMOVE:
		{
			//hent musekoordinater
			currMouseX = (int)LOWORD(lParam);
			currMouseY = (int)HIWORD(lParam);

			//og hvilken knapp som er trykket
			currMouseButton = (int)wParam;

			//roter om y aksen
			if ((currMouseY-prevMouseY > 3 || currMouseX-prevMouseX > 3) && (currMouseButton == MK_LBUTTON))
			{
				yv += (float)0.1f * ((float)currMouseY-prevMouseY) * mouseSensitivity; // / (float)s.Height);
			}

			//roter om x aksen
			if ((currMouseY-prevMouseY > 3 || currMouseX-prevMouseX > 3) && (currMouseButton == MK_RBUTTON))
			{
				xv += (float)0.1f * ((float)currMouseY-prevMouseY) * mouseSensitivity; // / (float)s.Height);
			}
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}