
#include "globals.h"
#include "perlin.h"

float noise2D(int x, int y)
{
	int n;
	float res;

	n = x + y * 57;
	n = (n<<13) ^ n;
	res = (float)( 1.0 - ( (n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff ) / 1073741824.0);

	return res;
}

float interpolate(float x, float y, float a) 
{
	float fac1 = (float)(3*((1-a)*(1-a)) - 2*((1-a)*(1-a)*(1-a)));
	float fac2 = (float)(3*(a*a) - 2*(a*a*a));

	return x*fac1 + y*fac2; //add the weighted factors
}

float getValue(float x, float y, int noiseFunc) 
{
	int Xint = (int)x;
	int Yint = (int)y;
	float Xfrac = x - Xint;
	float Yfrac = y - Yint;

	float x0y0;
	float x1y0;
	float x0y1;
	float x1y1;

	float xCount = randfloat(1.0f, 45.0f);
	float angle = randfloat(5.0f, 15.0f);
	float noisePower = randfloat(2.0f, 12.0f);

	if (noiseFunc == 1)
	{
		//noise(x, y)
		x0y0 = noise2D(Xint, Yint);  //finn styverdier for de fire hjrnene
		x1y0 = noise2D(Xint+1, Yint);
		x0y1 = noise2D(Xint, Yint+1);
		x1y1 = noise2D(Xint+1, Yint+1);
	}
	if (noiseFunc == 2)
	{
		//sin(x + 1 / |noise(x, y)|)
		x0y0 = sin(x + 1 / abs(noise2D(Xint, Yint))); //finn styverdier for de fire hjrnene
		x1y0 = sin(x + 1 / abs(noise2D(Xint+1, Yint)));
		x0y1 = sin(x + 1 / abs(noise2D(Xint, Yint+1)));
		x1y1 = sin(x + 1 / abs(noise2D(Xint+1, Yint+1)));
	}
	if (noiseFunc == 3)
	{
		//cos(PI *(x*xC) + (y*yC) * nP * noise(x, y))
		x0y0 = (float)(cos(PI * (x/xCount) + (y/angle) + (noisePower * noise2D(Xint, Yint))));  //finn styverdier for de fire hjrnene
		x1y0 = (float)(cos(PI * (x/xCount) + (y/angle) + (noisePower * noise2D(Xint+1, Yint))));
		x0y1 = (float)(cos(PI * (x/xCount) + (y/angle) + (noisePower * noise2D(Xint, Yint+1))));
		x1y1 = (float)(cos(PI * (x/xCount) + (y/angle) + (noisePower * noise2D(Xint+1, Yint+1))));
	}

	//interpolate between those values according to the x and y fractions
	float v1 = interpolate(x0y0, x1y0, Xfrac); //interpoler i x retningen (y)
	float v2 = interpolate(x0y1, x1y1, Xfrac); //interpoler i x retningen (y+1)
	float fin = interpolate(v1, v2, Yfrac);  //interpoler i y retningen

	return fin;
}

void makePerlinMap(float freq, float pers, float octaves, int noiseFunc)
{
	int c = 0;

	for(int i=0; i<MAP_SIZE; i++) 
	{
		for(int j=0; j<MAP_SIZE; j++) 
		{
			float total = 0;
			float amplitude = 1;
			float freq_factor = freq;

			for(int k=0; k<octaves; k++) 
			{
				total += getValue(j*freq_factor, i*freq_factor, noiseFunc) * amplitude;
				amplitude *= pers;
				freq_factor *= 2;
			}
			total = total*0.5f + 0.5f;

			if(total<0) total = 0.0f;
			if(total>1) total = 1.0f;
			int fin = (int)(total*255.0f); //kom innenfor fargerangen (0-255)

			g_HeightMap[c] = (BYTE)fin;
			c++;
		}
	}
}



