
#include "globals.h"
#include <math.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>
#include <gl\glaux.h>

float			sphere[37][37][3];							//sfren som skal damme sky domen
extern bool		bRenderSolid;	

//returnerer hyden p et gitt punkt i heightmap'n
int height(BYTE *pHeightMap, int X, int Y)
{
	int x = X % MAP_SIZE;
	int y = Y % MAP_SIZE;

	//sjekk om vi har et gyldig heightmap
	if(!pHeightMap) return 0;

	//indexer inn i heightmap'n og returner hyden p punktet
	return pHeightMap[x + (y * MAP_SIZE)];
}
												
//setter farge p en vertex avhengig av hydeindexen
void setVertexColor(BYTE *pHeightMap, int x, int y)
{
	//sjekk om vi har et gyldig heightmap
	if(!pHeightMap) return;

	float fColor = -0.15f + (height(pHeightMap, x, y ) / 256.0f);

	glColor3f(fColor/(fColor+50.0f), fColor, 0.05f);
}

void renderHeightMap(BYTE pHeightMap[])
{
	//disse brukes til  plye arrayet
	int X = 0, Y = 0;

	//disse brukes for  gjre ting lettere  se/lese
	int x, y, z;

	//sjekk om vi har et gyldig heightmap
	if(!pHeightMap) return;
		

	for ( X = 0; X < MAP_SIZE; X += STRIDE )
	{
		if(bRenderSolid)
		{
			glBegin(GL_QUADS);
		}
		else
		{
			glBegin(GL_LINE_STRIP);
		}

		for ( Y = 0; Y < MAP_SIZE; Y += STRIDE )
		{
			//x, y og z verdier for vertex nede til venstre
			x = X;							
			y = height(pHeightMap, X, Y );	
			z = Y;							

			//sett farge avhengig av hydeverdi
			setVertexColor(pHeightMap, x, z);

			//send vertex til OpenGL for tegning
			glVertex3i(x, y, z);

			//x, y og z verdier for vertex oppe til venstre
			x = X;										
			y = height(pHeightMap, X, Y + STRIDE );
			z = Y + STRIDE ;							
			
			//sett farge avhengig av hydeverdi
			setVertexColor(pHeightMap, x, z);

			//send vertex til OpenGL for tegning
			glVertex3i(x, y, z);

			//x, y og z verdier for vertex oppe til hyre
			x = X + STRIDE; 
			y = height(pHeightMap, X + STRIDE, Y + STRIDE ); 
			z = Y + STRIDE ;

			//sett farge avhengig av hydeverdi
			setVertexColor(pHeightMap, x, z);
			
			//send vertex til OpenGL for tegning
			glVertex3i(x, y, z);

			//x, y og z verdier for vertex nede til hyre
			x = X + STRIDE; 
			y = height(pHeightMap, X + STRIDE, Y ); 
			z = Y;

			//sett farge avhengig av hydeverdi
			setVertexColor(pHeightMap, x, z);

			//send vertex til OpenGL for tegning
			glVertex3i(x, y, z);
		}
		glEnd();

	}
	//resett fargene
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

GLvoid setSkyDomeData(float maxrad, float stride, float area, float maxheight)
{
	//maxradius av ringene
	float rad = ((float)maxrad*stride)/area;

	//setter tmp til maxverdi (37 ringer)
	float tmp = 37.0f;

	for (int i = 0; i < 37; i++)
	{
		for (int j = 0; j < 37; j++)
		{
			//x verdi
			sphere[i][j][0] = ((float)cos(2*PI*((double)(j*10)/360)))*(float)i/37*rad;
			//y verdi
			sphere[i][j][1] = ((float)sin(2*PI*((double)(j*10)/360)))*(float)i/37*rad;
			//Setter z verdi til log(tmp)
			sphere[i][j][2] = (float)log(tmp)*maxheight;
		}
		//trekker fra 1 p tmp (ned til 0)
		tmp--;
	}
}

GLvoid drawSky()
{
	for (int x = 0; x < 36; x++)
	{
		if (bRenderSolid)
		{
			glBegin(GL_TRIANGLE_STRIP);
		}
		else
		{
			glBegin(GL_LINE_STRIP);
		}

		for (int y = 0; y < 37; y++)
		{
			glVertex3f(sphere[x][y][0], sphere[x][y][1], sphere[x][y][2]);
			glVertex3f(sphere[x+1][y][0], sphere[x+1][y][1], sphere[x+1][y][2]);
		}
		glEnd();
	}
}

